/* www.N-Modellbahn.de
  Created by S.Schwarz
  Stellpult control with Millis Methode and INPUT_PULL
  Crossing light control with Millis Methode, pushbutton flip flop and INPUT_PULL

  History:
  V3: Einbau Flankenschutz Weiche 3 bei schalten Weiche 2 in gerade Richtung
  V3.1: DelayTimeWeiche angepasst von 500 auf 2000 um Hofmann Weichenantriebe anzusteuern
  V4: Signal control with I2C communication to track indicator integrated
  
*/

#include <EEPROM.h>
#include <Servo.h>

//##########################  I2C Communication  ######################################

#include <Wire.h>

/* MASTER */
#define NODE_MAX_NUMBER 2
#define PAYLOAD_SIZE 8
int nodePayload[NODE_MAX_NUMBER][PAYLOAD_SIZE];

//##########################  Signal control  ######################################

// **********************Signal light PIN********************

//Definition of input PIN for Signal light control pushbutton
const int  Button_inSig1 = 27;                        // the number of the signal 1 pushbutton pin
const int  Button_inSig2 = 28;                        // the number of the signal 2 pushbutton pin
const int  Button_inSig3 = 29;                        // the number of the signal 3 pushbutton pin
const int  Button_inSig4 = 30;                        // the number of the signal 4 pushbutton pin

// Definition of output PINs for Signal light LEDs:
const int Led_outSig1Red = 44;                       // the number of the Signal light 1 red LED pin
const int Led_outSig1Green = 45;                     // the number of the Signal light 1 green LED pin
const int Led_outSig2Red = 46;                       // the number of the Signal light 2 red LED pin
const int Led_outSig2Green = 47;                     // the number of the Signal light 2 green LED pin
const int Led_outSig3Red = 48;                       // the number of the Signal light 3 red LED pin
const int Led_outSig3Green = 49;                     // the number of the Signal light 3 green LED pin
const int Led_outSig4Red = 50;                       // the number of the Signal light 4 red LED pin
const int Led_outSig4Green = 51;                     // the number of the Signal light 4 green LED pin

// **********************Variables for Signal Control********************
int Button_stSig1;                                  // variable for reading the Signal 1 pushbutton status
int ButtonLast_stSig1 = HIGH;                       // variable for previous reading from the Signal 1 pushbutton
int Button_stReadingSig1;                           // variable for reading the Signal 1 pushbutton status
int SignalState_stSig1 = LOW;                       // Current state of the Signal 1 LOW = Hp0 & HIGH Hp1
int State_stSig1 = LOW;                             // Signal switching request LOW = Hp0 & HIGH = Hp1

int Button_stSig2;                                  // variable for reading the Signal 2 pushbutton status
int ButtonLast_stSig2 = HIGH;                       // variable for previous reading from the Signal 2 pushbutton
int Button_stReadingSig2;                           // variable for reading the Signal 2 pushbutton status
int SignalState_stSig2 = LOW;                       // Current state of the Signal 1 LOW = Hp0 & HIGH Hp1
int State_stSig2 = LOW;                             // Signal switching status Status LOW = HP0 & HIGH = HP1

int Button_stSig3;                                  // variable for reading the Signal 3 pushbutton status
int ButtonLast_stSig3 = HIGH;                       // variable for previous reading from the Signal 3 pushbutton
int Button_stReadingSig3;                           // variable for reading the Signal 3 pushbutton status
int SignalState_stSig3 = LOW;                       // Current state of the Signal 1 LOW = Hp0 & HIGH Hp1
int State_stSig3 = LOW;                             // Signal switching status Status LOW = HP0 & HIGH = HP1

int Button_stSig4;                                  // variable for reading the Signal 4 pushbutton status
int ButtonLast_stSig4 = HIGH;                       // variable for previous reading from the Signal 4 pushbutton
int Button_stReadingSig4;                           // variable for reading the Signal 4 pushbutton status
int SignalState_stSig4 = LOW;                       // Current state of the Signal 1 LOW = Hp0 & HIGH Hp1
int State_stSig4 = LOW;                             // Signal switching status Status LOW = HP0 & HIGH = HP1

// **********************Variables for Signal Timers********************
unsigned long Time_tiLastDebounceTime_Sig1 = 0;                             //variable for last time the Signal 1 pushbutton status change
unsigned long Time_tiLastDebounceTime_Sig2 = 0;                             //variable for last time the Signal 2 pushbutton status change
unsigned long Time_tiLastDebounceTime_Sig3 = 0;                             //variable for last time the Signal 3 pushbutton status change
unsigned long Time_tiLastDebounceTime_Sig4 = 0;                             //variable for last time the Signal 4 pushbutton status change
unsigned long Time_tiDebouncingSig = 50;                                    //debouncing time for Singal pushbutton


// ********************** Varibles for Track occupancy status ********************
int Led_stTrack4;
int Led_stTrack6;


//##########################  Crossing control  ######################################

// **********************Crossing light PIN********************

//Definition of input PIN for Crossing light control pushbutton
const int  Button_inCrossing = 31;                        // the number of the crossing light pushbutton pin

// Definition of output PINs for Crossing light LEDs:
const int Led_outCrossing1 =  52;                       // the number of the Crossing light 1 LED pin
const int Led_outCrossing2 =  53;                       // the number of the Crossing light 1 LED pin

// **********************Variables for Crossing light LED status********************
int Led_stCrossing1 = LOW;                            // variable for Crossing light 1 LED status
int Led_stCrossing2 = LOW;                            // variable for Crossing light 2 LED status

// **********************Variables for Crossing light pushbutton status********************
int Button_stCrossing = 0;                            // variable for reading the Crossing pushbutton status
int Blink_stCrossing = 1;                             // Variable for Crossing Blinking status
int Button_stCrossing_Previous = 0;                   // variable for storing last Crossing pushbutton satus

// **********************Variables for timers********************
unsigned long Crossing_tiCurrentMillis = 0;           // variable for reading the current millis value
unsigned long Crossing_tiPreviousMillis = 0;          // variable to store the previous millis value
unsigned long Crossing_tiLightInterval = 500;         // Interval timer for Crossing light flashing



//########################  Deflector control  #########################################

// **********************Deflector PIN********************

// Definition of constants. They're used here to set pin numbers:
//Definition of input PINs for Deflector control pushbutton
const int Button_inStartLft = 22;      // the number of the Start track left pushbutton pin
const int Button_inStartRgt = 23;      // the number of the Start track right pushbutton pin
const int Button_inGleis1 = 24;        // the number of the track 1 pushbutton pin
const int Button_inGleis2 = 25;        // the number of the track 2 pushbutton pin
const int Button_inGleis3 = 26;        // the number of the track 3 pushbutton pin

// Definition of output PINs for Deflector indication LEDs:
const int Led_outWeiche1Straight = 32;   // the number of the Deflector 1 Straight LED pin
const int Led_outWeiche1Turn = 33;       // the number of the Deflector 1 Turn LED pin
const int Led_outWeiche2Straight = 34;   // the number of the Deflector 2 Straight LED pin
const int Led_outWeiche2Turn = 35;       // the number of the Deflector 2 Turn LED pin
const int Led_outWeiche3Straight = 36;   // the number of the Deflector 3 Straight LED pin
const int Led_outWeiche3Turn = 37;       // the number of the Deflector 3 Turn LED pin

// Definition of output PINs for Deflector Relays:
const int Relay_outWeiche1Straight = 38;     // the number of the Deflector 1 Straight Relay pin
const int Relay_outWeiche1Turn = 39;         // the number of the Deflector 1 Turn Relay pin
const int Relay_outWeiche2Straight = 40;     // the number of the Deflector 2 Straight Relay pin
const int Relay_outWeiche2Turn = 41;         // the number of the Deflector 2 Turn Relay pin
const int Relay_outWeiche3Straight = 42;     // the number of the Deflector 3 Straight Relay pin
const int Relay_outWeiche3Turn = 43;         // the number of the Deflector 3 Turn Relay pin

// **********************Variables for Pushbutton status********************

// variables for reading Deflector pushbutton status:
int Button_stStartLft = 1;              // variable for reading the Start track left pushbutton
int Button_stStartRgt = 1;              // variable for reading the Start track right pushbutton
int Button_stGleis1 = 1;                // variable for reading the track 1 pushbutton
int Button_stGleis2 = 1;                // variable for reading the track 2 pushbutton
int Button_stGleis3 = 1;                // variable for reading the track 3 pushbutton

// **********************Variables for LED status********************
// variables for reading Deflector LED status:
int Led_stWeiche1Straight = 0;              // variable for reading the Deflector 1 Straight LED pin
int Led_stWeiche1Turn = 0;                  // variable for reading the Deflector 1 Turn LED pin
int Led_stWeiche2Straight = 0;              // variable for reading the Deflector 2 Straight LED pin
int Led_stWeiche2Turn = 0;                  // variable for reading the Deflector 2 Turn LED pin
int Led_stWeiche3Straight = 0;              // variable for reading the Deflector 3 Straight LED pin
int Led_stWeiche3Turn = 0;                  // variable for reading the Deflector 3 Turn LED pin

// **********************Variables for LED status********************
int IfPath_st1 = 0;                           // variable for reading ifPath status
int IfPath_st2 = 0;                           // variable for reading ifPath status
int IfPath_st3 = 0;                           // variable for reading ifPath status
int IfPath_st4 = 0;                           // variable for reading ifPath status
int IfPath_st5 = 0;                           // variable for reading ifPath status
int IfPath_st6 = 0;                           // variable for reading ifPath status
int IfPath_st7 = 0;                           // variable for reading ifPath status
int IfPath_st8 = 0;                           // variable for reading ifPath status

// **********************Variables for timers********************
// variables for reading the millis () value
unsigned long  Time_tilast1 = 0;               // variable for reading the millis value 1
unsigned long  Time_tilast2 = 0;               // variable for reading the millis value 2
unsigned long  Time_tilast3 = 0;               // variable for reading the millis value 3
unsigned long  Time_tilast4 = 0;               // variable for reading the millis value 4
unsigned long  Time_tilast5 = 0;               // variable for reading the millis value 5
unsigned long  Time_tilast6 = 0;               // variable for reading the millis value 6
unsigned long  Time_tilast7 = 0;               // variable for reading the millis value 7
unsigned long  Time_tilast8 = 0;               // variable for reading the millis value 7

const int DelayTimeWeiche = 2000;               // Delay timer duration
const int DelayTimeServo = 100;                //Delay time duration to control servo speed

// **********************Variables for EEPROM********************
int EEprom_id0 = 0;
int EEprom_id1 = 1;
int EEprom_id2 = 2;
int EEprom_id3 = 3;
int EEprom_id4 = 4;
int EEprom_id5 = 5;
int EEprom_id6 = 6;
int EEprom_id7 = 7;
int EEprom_id8 = 8;
int EEprom_id9 = 9;
int EEprom_id10 = 10;
int EEprom_id11 = 11;


void setup() {
  //##########################  I2C Communication  ######################################
   Wire.begin();
   Serial.begin(9600); 
  
  //##########################  Signal control  ######################################

  //*******************Signal Control PIN Mode******************

  // initialize the Signal pushbutton pin as an input:
  pinMode(Button_inSig1, INPUT_PULLUP);
  pinMode(Button_inSig2, INPUT_PULLUP);
  pinMode(Button_inSig3, INPUT_PULLUP);
  pinMode(Button_inSig4, INPUT_PULLUP);

  // initialize the Signal Light LED pins as an output:
  pinMode(Led_outSig1Red, OUTPUT);
  pinMode(Led_outSig1Green, OUTPUT);
  pinMode(Led_outSig2Red, OUTPUT);
  pinMode(Led_outSig2Green, OUTPUT);
  pinMode(Led_outSig3Red, OUTPUT);
  pinMode(Led_outSig3Green, OUTPUT);
  pinMode(Led_outSig4Red, OUTPUT);
  pinMode(Led_outSig4Green, OUTPUT);

  // Initialize the Signal Light LED Status:
  digitalWrite(Led_outSig1Red, HIGH);
  digitalWrite(Led_outSig1Green, LOW);
  digitalWrite(Led_outSig2Red, HIGH);
  digitalWrite(Led_outSig2Green, LOW);  
  digitalWrite(Led_outSig3Red, HIGH);
  digitalWrite(Led_outSig3Green, LOW);
  digitalWrite(Led_outSig4Red, HIGH);
  digitalWrite(Led_outSig4Green, LOW);
    
  //##########################  Crossing control  ######################################

  //*******************Crossing PIN Mode******************

  // initialize the Crossing Light pushbutton pin as an input:
  pinMode(Button_inCrossing, INPUT_PULLUP);

  // initialize the Crossing Light LED pins as an output:
  pinMode(Led_outCrossing1, OUTPUT);
  pinMode(Led_outCrossing2, OUTPUT);

  //########################  Deflector control  #########################################

  // initialize the Deflector pushbutton pin as an input:
  pinMode(Button_inStartLft, INPUT_PULLUP);
  pinMode(Button_inStartRgt, INPUT_PULLUP);
  pinMode(Button_inGleis1, INPUT_PULLUP);
  pinMode(Button_inGleis2, INPUT_PULLUP);
  pinMode(Button_inGleis3, INPUT_PULLUP);

  // initialize the Deflector LED pins as an output:
  pinMode(Led_outWeiche1Straight, OUTPUT);
  pinMode(Led_outWeiche1Turn, OUTPUT);
  pinMode(Led_outWeiche2Straight, OUTPUT);
  pinMode(Led_outWeiche2Turn, OUTPUT);
  pinMode(Led_outWeiche3Straight, OUTPUT);
  pinMode(Led_outWeiche3Turn, OUTPUT);

  // initialize the Deflector Relay pins as an output:
  pinMode(Relay_outWeiche1Straight, OUTPUT);
  pinMode(Relay_outWeiche1Turn, OUTPUT);
  pinMode(Relay_outWeiche2Straight, OUTPUT);
  pinMode(Relay_outWeiche2Turn, OUTPUT);
  pinMode(Relay_outWeiche3Straight, OUTPUT);
  pinMode(Relay_outWeiche3Turn, OUTPUT);

  //initialize the Deflector Relay pins status:
  digitalWrite(Relay_outWeiche1Straight, HIGH);
  digitalWrite(Relay_outWeiche1Turn, HIGH);
  digitalWrite(Relay_outWeiche2Straight, HIGH);
  digitalWrite(Relay_outWeiche2Turn, HIGH);
  digitalWrite(Relay_outWeiche3Straight, HIGH);
  digitalWrite(Relay_outWeiche3Turn, HIGH);


  //*******************Init LED Status from EEPROM******************
  checkLedState();

}

void loop() {
  //##########################  I2C Communication to track indicator  ######################################
   for (int i=1; i<=NODE_MAX_NUMBER; i++){          // Start to request Data
     receiveData(i);                                // Start of subroutine to get Data from I2C
     }   
 
    Led_stTrack4 = nodePayload[1] [4];
    Led_stTrack6 = nodePayload[1] [6];

  
//##########################  Signal control  ######################################  
  
  //*******************Signal 1 Button Control ******************
  
  Button_stReadingSig1 = digitalRead (Button_inSig1);
  if (Button_stReadingSig1 != ButtonLast_stSig1) {
    Time_tiLastDebounceTime_Sig1 = millis();                                    // reset the debouncing timer in case the Status of the pushbutton has changed
  }
  if (((millis() - Time_tiLastDebounceTime_Sig1) > Time_tiDebouncingSig)) {      // check if debouncing time to is elapsed
    
    if (Button_stReadingSig1 != Button_stSig1) {                               // check if push button status change is still present after debouncing time 
        Button_stSig1 = Button_stReadingSig1;
      
      if ((Button_stSig1 == LOW)&&(Led_stWeiche1Turn == 1)&&(Led_stTrack4==LOW)&&(SignalState_stSig1==LOW)) {                                             // only change LED state if the new pushbutton was pressed
        State_stSig1 = !State_stSig1;
        }
    if ((Button_stSig1 == LOW)&&(SignalState_stSig1 ==HIGH)) {                                             // only change LED state if the new pushbutton was pressed
        State_stSig1 = !State_stSig1;
        }
      }
    }
  
  if (((Led_stTrack4==HIGH))&&(SignalState_stSig1 ==HIGH)){
    State_stSig1 = LOW;
  }
  
  
  if (State_stSig1 == LOW) {
    digitalWrite(Led_outSig1Red, HIGH);                                       // Switch red Signal LED ON or OFF
    digitalWrite(Led_outSig1Green, LOW);                                      // Switch green Signal LED ON or OFF
    SignalState_stSig1 =LOW;
  }
  if (State_stSig1 == HIGH) {
    digitalWrite(Led_outSig1Red, LOW);                                        // Switch red Signal LED ON or OFF
    digitalWrite(Led_outSig1Green, HIGH);                                     // Switch green Signal LED ON or OFF
    SignalState_stSig1 =HIGH;
  }

  ButtonLast_stSig1 = Button_stReadingSig1;                                   // save the Reading status of the Pushbutton Next time through the loop, it'll be the lastButtonState

//*******************Signal 2 Button Control ******************
  
  Button_stReadingSig2 = digitalRead (Button_inSig2);
  if (Button_stReadingSig2 != ButtonLast_stSig2) {
    Time_tiLastDebounceTime_Sig2 = millis();                                    // reset the debouncing timer in case the Status of the pushbutton has changed
  }
  if (((millis() - Time_tiLastDebounceTime_Sig2) > Time_tiDebouncingSig)) {      // check if debouncing time to is elapsed
    
    if (Button_stReadingSig2 != Button_stSig2) {                               // check if push button status change is still present after debouncing time 
        Button_stSig2 = Button_stReadingSig2;
      
      if ((Button_stSig2 == LOW)&&(Led_stWeiche3Turn == 1)&&(Led_stWeiche2Turn == 1)&&(Led_stTrack6==LOW)&&(SignalState_stSig2==LOW)) {                                             // only change LED state if the new pushbutton was pressed //&& (Button_stReadingSig2==HIGH)&&(State_stSig1==LOW))
        State_stSig2 = !State_stSig2;
        }
    if ((Button_stSig2 == LOW)&&(SignalState_stSig2 ==HIGH)) {                                             // only change LED state if the new pushbutton was pressed
        State_stSig2 = !State_stSig2;
        }
      }
    }
  
  if (((Led_stTrack6==HIGH))&&(SignalState_stSig2 ==HIGH)){
    State_stSig2 = LOW;
  }
  
  
  if (State_stSig2 == LOW) {
    digitalWrite(Led_outSig2Red, HIGH);                                       // Switch red Signal LED ON or OFF
    digitalWrite(Led_outSig2Green, LOW);                                      // Switch green Signal LED ON or OFF
    SignalState_stSig2 =LOW;
  }
  if (State_stSig2 == HIGH) {
    digitalWrite(Led_outSig2Red, LOW);                                        // Switch red Signal LED ON or OFF
    digitalWrite(Led_outSig2Green, HIGH);                                     // Switch green Signal LED ON or OFF
    SignalState_stSig2 =HIGH;
  }

  ButtonLast_stSig2 = Button_stReadingSig2;                                   // save the Reading status of the Pushbutton Next time through the loop, it'll be the lastButtonState

 //*******************Signal 3 Button Control ******************
  
  Button_stReadingSig3 = digitalRead (Button_inSig3);
  if (Button_stReadingSig3 != ButtonLast_stSig3) {
    Time_tiLastDebounceTime_Sig3 = millis();                                    // reset the debouncing timer in case the Status of the pushbutton has changed
  }
  if (((millis() - Time_tiLastDebounceTime_Sig3) > Time_tiDebouncingSig)) {      // check if debouncing time is elapsed
    
    if (Button_stReadingSig3 != Button_stSig3) {                               // check if push button status change is still present after debouncing time 
        Button_stSig3 = Button_stReadingSig3;
      
      if ((Button_stSig3 == LOW)&&(Led_stWeiche1Straight == 1)&&(Led_stTrack4==LOW)&&(SignalState_stSig3==LOW)) {                                             // change to Hp1 Signal state (High) if the new pushbutton was pressed and Weiche 1 is in straight position and Track 4 is free and current Signal status is Hp0
        State_stSig3 = !State_stSig3;
        }
    if ((Button_stSig3 == LOW)&&(SignalState_stSig3 ==HIGH)) {                                             //Change to Hp0 state (low) if new pushbutton was pressed and current signal status is Hp1
        State_stSig3 = !State_stSig3;
        }
      }
    }
  
  if (((Led_stTrack4==HIGH))&&(SignalState_stSig3 ==HIGH)){                                               //Change to Hp0 state (low) if track4 is blocked by a train and current singnal status is Hp1
    State_stSig3 = LOW;
  }
  
  
  if (State_stSig3 == LOW) {
    digitalWrite(Led_outSig3Red, HIGH);                                       // Switch red Signal LED ON or OFF
    digitalWrite(Led_outSig3Green, LOW);                                      // Switch green Signal LED ON or OFF
    SignalState_stSig3 =LOW;
  }
  if (State_stSig3 == HIGH) {
    digitalWrite(Led_outSig3Red, LOW);                                        // Switch red Signal LED ON or OFF
    digitalWrite(Led_outSig3Green, HIGH);                                     // Switch green Signal LED ON or OFF
    SignalState_stSig3 =HIGH;
  }

  ButtonLast_stSig3 = Button_stReadingSig3;                                   // save the Reading status of the Pushbutton Next time through the loop, it'll be the lastButtonState


 //*******************Signal 4 Button Control ******************
  
  Button_stReadingSig4 = digitalRead (Button_inSig4);
  if (Button_stReadingSig4 != ButtonLast_stSig4) {
    Time_tiLastDebounceTime_Sig4 = millis();                                    // reset the debouncing timer in case the Status of the pushbutton has changed
  }
  if (((millis() - Time_tiLastDebounceTime_Sig4) > Time_tiDebouncingSig)) {      // check if debouncing time is elapsed
    
    if (Button_stReadingSig4 != Button_stSig4) {                               // check if push button status change is still present after debouncing time 
        Button_stSig4 = Button_stReadingSig4;
      
      if ((Button_stSig4 == LOW)&&(Led_stWeiche2Straight == 1)&&(Led_stTrack6==LOW)&&(SignalState_stSig4==LOW)) {                                             // change to Hp1 Signal state (High) if the new pushbutton was pressed and Weiche 2 is in straight position and Track 6 is free and current Signal status is Hp0
        State_stSig4 = !State_stSig4;
        }
    if ((Button_stSig4 == LOW)&&(SignalState_stSig4 ==HIGH)) {                                             //Change to Hp0 state (low) if new pushbutton was pressed and current signal status is Hp1
        State_stSig4 = !State_stSig4;
        }
      }
    }
  
  if (((Led_stTrack6==HIGH))&&(SignalState_stSig4 ==HIGH)){                                               //Change to Hp0 state (low) if track6 is blocked by a train and current singnal status is Hp1
    State_stSig4 = LOW;
  }
  
  
  if (State_stSig4 == LOW) {
    digitalWrite(Led_outSig4Red, HIGH);                                       // Switch red Signal LED ON or OFF
    digitalWrite(Led_outSig4Green, LOW);                                      // Switch green Signal LED ON or OFF
    SignalState_stSig4 =LOW;
  }
  if (State_stSig4 == HIGH) {
    digitalWrite(Led_outSig4Red, LOW);                                        // Switch red Signal LED ON or OFF
    digitalWrite(Led_outSig4Green, HIGH);                                     // Switch green Signal LED ON or OFF
    SignalState_stSig4 =HIGH;
  }

  ButtonLast_stSig4 = Button_stReadingSig4;                                   // save the Reading status of the Pushbutton Next time through the loop, it'll be the lastButtonState
  
  
  
  //##########################  Crossing control  ######################################

  
  // ****************** Crossing LED control: **********************
 
  Crossing_tiCurrentMillis = millis();                                                            // Write actual time in crossing_tiCurrentMillis

  if ( Blink_stCrossing == HIGH) {                                                                // check if Blink_stCrossing is active, if yes activate crossing LEDs
    if (Crossing_tiCurrentMillis - Crossing_tiPreviousMillis > Crossing_tiLightInterval) {        // check if delay time to switch other LEDs is elapsed
      Crossing_tiPreviousMillis = Crossing_tiCurrentMillis;                                       // write actual time to Crossing_tiPreviousMillis
      if (Led_stCrossing1 == LOW) {                                                               // check if LED status is LOW
        Led_stCrossing1 = HIGH;                                                                   // Set Led_stCrossing1 to High
        Led_stCrossing2 = LOW;                                                                    // Set Led_stCrossing2 to low
      } else {
        Led_stCrossing1 = LOW;                                                                    // Set Led_stCrossing1 to low
        Led_stCrossing2 = HIGH;                                                                   // Set Led_stCrossing2 to High
      }
      digitalWrite(Led_outCrossing1, Led_stCrossing1);                                            // Switch LED Crossing 1 corresponding to Led_stCrossing1)
      digitalWrite(Led_outCrossing2, Led_stCrossing2);                                            // Switch LED Crossing 2 corresponding to Led_stCrossing2)
    }
  } else {
    digitalWrite(Led_outCrossing1, LOW);                                                          // Switch LED Crossing 1 off                                                      
    digitalWrite(Led_outCrossing2, LOW);                                                          // Switch LED Crossing 2 off  
  }

  // ************** Check Crossing pushbutton status for Flip Flop:***************
  Button_stCrossing = digitalRead (Button_inCrossing);                                           // Check crossing pushbutton Status
  if (Button_stCrossing_Previous != Button_stCrossing) {                                         // Check if crossing pushbutton Status has changed
    if (Button_stCrossing == HIGH)                                                               // If pushbutton was activated, Blink_stCrossing will be changed
    {

      Blink_stCrossing = !Blink_stCrossing;                 

    }
    Button_stCrossing_Previous = Button_stCrossing;                                               // Actual Crossing pushbutton Status is stored
    delay (200);                                                                                  // Debouncing for the pushbutton to avoid toggle of signal   
  }

  //########################  Deflector control  #########################################

  // ########### read the state of the Deflector pushbutton value: ###########
  Button_stStartLft = digitalRead(Button_inStartLft);
  Button_stStartRgt = digitalRead(Button_inStartRgt);
  Button_stGleis1 = digitalRead(Button_inGleis1);
  Button_stGleis2 = digitalRead(Button_inGleis2);
  Button_stGleis3 = digitalRead(Button_inGleis3);


  // ########### Deflector control: ###########

  if ((Button_stStartLft == LOW) && (Button_stGleis1 == LOW))      // check if the Start left & Gleis 1 Button is pressed
  {
    digitalWrite(Relay_outWeiche1Turn, LOW);                      // Switch Relay ON for Weiche 1 Turn position
    digitalWrite(Led_outWeiche1Turn, HIGH);                       // Switch LED ON for Weiche 1 Turn position indication
    digitalWrite(Led_outWeiche1Straight, LOW);                    // Switch LED OFF for Weiche 1 Straight position indication
    Led_stWeiche1Straight = 0;                                    // Status LED Weiche 1 Straight position indication for EEPROM
    Led_stWeiche1Turn = 1;                                        // Status LED Weiche 1 Turn position indication for EEPROM
    EEPROM.update(EEprom_id0, Led_stWeiche1Straight);             // Status LED Weiche 1 Straight position stored in EEPROM if value different to already stored value
    EEPROM.update(EEprom_id1, Led_stWeiche1Turn);                 // Status LED Weiche 1 turned position stored in EEPROM if value different to already stored value
    Time_tilast1 = millis();                                      // Write actual time in Time_tilast1
    IfPath_st1 = 1;                                               // Write status of path = 1 (needed to ensure proper working of millis function)
  }
  if (((millis() - Time_tilast1) >= DelayTimeWeiche) && (IfPath_st1 == 1))       //check if delay time to switch off the relay is elapsed & IfPath is true
  {
    digitalWrite(Relay_outWeiche1Turn, HIGH);                     // Switch Relay OFF for Weiche 1 Turn position
    IfPath_st1 = 0;                                                // Set IfPath back to 0
  }

  if ((Button_stStartLft == LOW) && (Button_stGleis2 == LOW))     // check if the Start left & Gleis 2 Button is pressed
  {
    digitalWrite(Relay_outWeiche1Straight, LOW);                  // Switch Relay ON for Weiche 1 Straight position
    digitalWrite(Led_outWeiche1Straight, HIGH);                   // Switch LED ON for Weiche 1 Straight position indication
    digitalWrite(Led_outWeiche1Turn, LOW);                        // Switch LED OFF for Weiche 1 Turn position indication
    Led_stWeiche1Straight = 1;                                    // Status LED Weiche 1 Straight position indication for EEPROM
    Led_stWeiche1Turn = 0;                                        // Status LED Weiche 1 Turn position indication for EEPROM
    EEPROM.update(EEprom_id0, Led_stWeiche1Straight);             // Status LED Weiche 1 Straight position stored in EEPROM if value different to already stored value
    EEPROM.update(EEprom_id1, Led_stWeiche1Turn);                 // Status LED Weiche 1 turned position stored in EEPROM if value different to already stored value
    Time_tilast2 = millis();                                      // Write actual time in Time_tilast2
    IfPath_st2 = 1;                                               // Write status of path = 1 (needed to ensure proper working of millis function)
  }
  if (((millis() - Time_tilast2) >= DelayTimeWeiche) && (IfPath_st2 == 1))            //check if delay time to switch off the relay is elapsed & IfPath is true
  {
    digitalWrite(Relay_outWeiche1Straight, HIGH);                 // Switch Relay OFF for Weiche 1 Straight position
    IfPath_st2 = 0;                                               // Set IfPath back to 0
  }

  if ((Button_stStartLft == LOW) && (Button_stGleis3 == LOW))     // check if the Start left & Gleis 3 Button is pressed
  {
    digitalWrite(Relay_outWeiche1Turn, LOW);                      // Switch Relay ON for Weiche 1 Turn position
    digitalWrite(Led_outWeiche1Turn, HIGH);                       // Switch LED ON for Weiche 1 Turn position indication
    digitalWrite(Led_outWeiche1Straight, LOW);                    // Switch LED OFF for Weiche 1 Straight position indication
    Led_stWeiche1Straight = 0;                                    // Status LED Weiche 1 Straight position indication for EEPROM
    Led_stWeiche1Turn = 1;                                        // Status LED Weiche 1 Turn position indication for EEPROM
    EEPROM.update(EEprom_id0, Led_stWeiche1Straight);             // Status LED Weiche 1 Straight position stored in EEPROM if value different to already stored value
    EEPROM.update(EEprom_id1, Led_stWeiche1Turn);                 // Status LED Weiche 1 turned position stored in EEPROM if value different to already stored value
    Time_tilast3 = millis();                                      // Write actual time in Time_tilast3
    IfPath_st3 = 1;                                               // Write status of path = 1 (needed to ensure proper working of millis function)
  }
  if (((millis() - Time_tilast3) >= DelayTimeWeiche) && (IfPath_st3 == 1))             //check if delay time to switch off the relay is elapsed
  {
    digitalWrite(Relay_outWeiche1Turn, HIGH);                     // Switch Relay OFF for Weiche 1 Straight position
    digitalWrite(Relay_outWeiche3Straight, LOW);                  // Switch Relay ON for Weiche 3 Straight position
    digitalWrite(Led_outWeiche3Straight, HIGH);                   // Switch LED ON for Weiche 3 Straight position indication
    digitalWrite(Led_outWeiche3Turn, LOW);                        // Switch LED OFF for Weiche 3 Turn position indication
    Led_stWeiche3Straight = 1;                                    // Status LED Weiche 3 Straight position indication for EEPROM
    Led_stWeiche3Turn = 0;                                        // Status LED Weiche 3 Turn position indication for EEPROM
    EEPROM.update(EEprom_id4, Led_stWeiche3Straight);             // Status LED Weiche 3 Straight position stored in EEPROM if value different to already stored value
    EEPROM.update(EEprom_id5, Led_stWeiche3Turn);                 // Status LED Weiche 3 turned position stored in EEPROM if value different to already stored value
    IfPath_st3 = 0;                                               // Set IfPath back to 0
    Time_tilast4 = millis();                                      // Write actual time in Time_tilast4
    IfPath_st4 = 1;                                               // Write status of path = 1 (needed to ensure proper working of millis function)
  }
  if (((millis() - Time_tilast4) >= DelayTimeWeiche) && (IfPath_st4 == 1))           //check if delay time to switch off the relay is elapsed
  {
    digitalWrite(Relay_outWeiche3Straight, HIGH);                 // Switch Relay OFF for Weiche 3 Straight position
    IfPath_st4 = 0;                                               // Set IfPath back to 0
  }

  if ((Button_stStartRgt == LOW) && (Button_stGleis1 == LOW))     // check if the Start right & Gleis 1 Button is pressed
  {
    digitalWrite(Relay_outWeiche2Turn, LOW);                      // Switch Relay ON for Weiche 2 Turn position
    digitalWrite(Led_outWeiche2Turn, HIGH);                       // Switch LED ON for Weiche 2 Turn position indication
    digitalWrite(Led_outWeiche2Straight, LOW);                    // Switch LED OFF for Weiche 2 Straight position indication
    Led_stWeiche2Straight = 0;                                    // Status LED Weiche 2 Straight position indication for EEPROM
    Led_stWeiche2Turn = 1;                                        // Status LED Weiche 2 Turn position indication for EEPROM
    EEPROM.update(EEprom_id2, Led_stWeiche2Straight);             // Status LED Weiche 2 Straight position stored in EEPROM if value different to already stored value
    EEPROM.update(EEprom_id3, Led_stWeiche2Turn);                 // Status LED Weiche 2 turned position stored in EEPROM if value different to already stored value
    Time_tilast5 = millis();                                      // Write actual time in Time_tilast5
    IfPath_st5 = 1;                                               // Write status of path = 1 (needed to ensure proper working of millis function)
  }
  if (((millis() - Time_tilast5) >= DelayTimeWeiche) && (IfPath_st5 == 1))             //check if delay time to switch off the relay is elapsed
  {
    digitalWrite(Relay_outWeiche2Turn, HIGH);                     // Switch Relay OFF for Weiche 2 Turn position
    digitalWrite(Relay_outWeiche3Turn, LOW);                      // Switch Relay ON for Weiche 3 Turn position
    digitalWrite(Led_outWeiche3Turn, HIGH);                       // Switch LED ON for Weiche 3 Turn position indication
    digitalWrite(Led_outWeiche3Straight, LOW);                    // Switch LED OFF for Weiche 3 Straight position indication
    Led_stWeiche3Straight = 0;                                    // Status LED Weiche 3 Straight position indication for EEPROM
    Led_stWeiche3Turn = 1;                                        // Status LED Weiche 3 Turn position indication for EEPROM
    EEPROM.update(EEprom_id4, Led_stWeiche3Straight);             // Status LED Weiche 3 Straight position stored in EEPROM if value different to already stored value
    EEPROM.update(EEprom_id5, Led_stWeiche3Turn);                 // Status LED Weiche 3 turned position stored in EEPROM if value different to already stored value
    IfPath_st5 = 0;                                               // Set IfPath back to 0
    Time_tilast6 = millis();                                      // Write actual time in Time_tilast6
    IfPath_st6 = 1;                                               // Write status of path = 1 (needed to ensure proper working of millis function)
  }
  if (((millis() - Time_tilast6) >= DelayTimeWeiche) && (IfPath_st6 == 1))           //check if delay time to switch off the relay is elapsed
  {
    digitalWrite(Relay_outWeiche3Turn, HIGH);                     // Switch Relay OFF for Weiche 2 Turn position
    IfPath_st6 = 0;                                               // Set IfPath back to 0
  }

 if ((Button_stStartRgt == LOW) && (Button_stGleis2 == LOW))     // check if the Start right & Gleis 2 Button is pressed
  {
    digitalWrite(Relay_outWeiche2Straight, LOW);                  // Switch Relay ON for Weiche 2 Straight position
    digitalWrite(Led_outWeiche2Straight, HIGH);                   // Switch LED ON for Weiche 2 Straight position indication
    digitalWrite(Led_outWeiche2Turn, LOW);                        // Switch LED OFF for Weiche 2 Turn position indication
    Led_stWeiche2Straight = 1;                                    // Status LED Weiche 2 Straight position indication for EEPROM
    Led_stWeiche2Turn = 0;                                        // Status LED Weiche 2 Turn position indication for EEPROM
    EEPROM.update(EEprom_id2, Led_stWeiche2Straight);             // Status LED Weiche 2 Straight position stored in EEPROM if value different to already stored value
    EEPROM.update(EEprom_id3, Led_stWeiche2Turn);                 // Status LED Weiche 2 turned position stored in EEPROM if value different to already stored value
    Time_tilast7 = millis();                                      // Write actual time in Time_tilast7
    IfPath_st7 = 1;                                               // Write status of path = 1 (needed to ensure proper working of millis function)
  }

  if (((millis() - Time_tilast7) >= DelayTimeWeiche) && (IfPath_st7 == 1)) //check if delay time to switch off the relay is elapsed
    {
    digitalWrite(Relay_outWeiche2Straight, HIGH);                     // Switch Relay OFF for Weiche 2 Turn position
    digitalWrite(Relay_outWeiche3Straight, LOW);                      // Switch Relay ON for Weiche 3 Turn position
    digitalWrite(Led_outWeiche3Straight, HIGH);                       // Switch LED ON for Weiche 3 Turn position indication
    digitalWrite(Led_outWeiche3Turn, LOW);                    // Switch LED OFF for Weiche 3 Straight position indication
    Led_stWeiche3Straight = 1;                                    // Status LED Weiche 3 Straight position indication for EEPROM
    Led_stWeiche3Turn = 0;                                        // Status LED Weiche 3 Turn position indication for EEPROM
    EEPROM.update(EEprom_id4, Led_stWeiche3Straight);             // Status LED Weiche 3 Straight position stored in EEPROM if value different to already stored value
    EEPROM.update(EEprom_id5, Led_stWeiche3Turn);                 // Status LED Weiche 3 turned position stored in EEPROM if value different to already stored value
    IfPath_st7 = 0;                                               // Set IfPath back to 0
    Time_tilast8 = millis();                                      // Write actual time in Time_tilast6
    IfPath_st8 = 1;                                               // Write status of path = 1 (needed to ensure proper working of millis function)
  }
  
   if (((millis() - Time_tilast8) >= DelayTimeWeiche) && (IfPath_st8 == 1))           //check if delay time to switch off the relay is elapsed   
  {
    digitalWrite(Relay_outWeiche3Straight, HIGH);                 // Switch Relay OFF for Weiche 2 Straight position
    IfPath_st8 = 0;                                               // Set IfPath back to 0
   }

}
